module Rosetree where

-- Die Datenstruktur
data Rose a = Node a [Rose a]
	deriving(Show)
	
data Btree a = Leaf a
	| Fork (Btree a) (Btree a)
	deriving(Show)

-- size:
-- Gre des Baumes mit map und sum
size :: Rose a -> Int
size (Node x xs) = 1 + sum( map size xs)

-- height:
-- Hhe des Rosenbaumes
height :: Rose a -> Int
height (Node x xs) = 1 + maxlist( map height xs)

-- depths:
-- Tiefe des Baumes
-- depths :: Rose a -> Rose Int
-- depths	= down 1
--	where
--	down :: Int -> Rose a -> Rose Int
--	down n (Node x xs) = Node n (map (down (n+1)) xs)

-- maxRose:
-- Das maximale Element eines Rosenbaumes
maxRose :: (Num a, Ord a) => Rose a -> a
maxRose (Node x xs) = x `max` maxlist (map maxRose xs)
	
-- height xt y= size xt <= (k ^(height xt))/(k-1)

-- mapRose:
-- Map-Funktion fr Rosenbume
mapRose :: (a -> b) -> Rose a -> Rose b
mapRose f (Node x xs) = Node (f x) (map (mapRose f) xs)

-- foldRose:
-- fold fr Rosenbaum
foldRose :: (a -> [b] -> b) -> Rose a -> b
foldRose f (Node x xs) = f x (map (foldRose f) xs)

-- size2 & height2
-- Hhe und Gre mit foldRose
size2    = foldRose (\x xs -> 1 + sum xs)
height2  = foldRose (\x xs -> 1 + maxlist xs)
flatten  = foldRose (\x xs -> [x] ++ (foldr (++) [] xs) )
maxRose2 = foldRose (\x xs -> (x `max` maxlist xs))

-- flatten:
-- Konvertieren eines Rosenbaumes in eine Liste mit foldRose
--flatten :: Rose a -> [a]
--flatten = foldRose f
	--where
	--f :: a -> [[a]] -> [a]
	--f a bs = [a] ++ (foldr (++) [] bs)

-- toB:	
-- Rosen- in Binrbaum konvertieren
toB :: Rose a -> Btree a
toB (Node a xs) = foldl Fork (Leaf a) (map toB xs)

-- toR:
-- Konvertieren eines Binrbaumes in einen Rosenbaum
toR :: Btree a -> Rose a
toR (Leaf x) = Node x []
toR (Fork xb yb) = Node x (xts++[xs])
	where
	Node x xts = toR xb
	xs = toR yb

-- Hilfsfunktion
getChildren :: Rose a -> [Rose a]
getChildren (Node a xs) = xs

maxlist :: (Num a, Ord a) => [a] -> a
maxlist = foldl (max) 0

from :: a -> [a]
from x = x : from x

from1 :: a -> Rose a
from1 x = Node x [from1 x]

from2 :: a -> Rose a
from2 x = Node x (from (Node x []))
	
-- Beispielbume
c1 = Node "a" []
c2 = Node "b" [c1,c1,c1]
c3 = Node "a" [(Node "b" [(Node "c" []),(Node "d" []),(Node "e" [])]),(Node "f" [(Node "g" [(Node "h" [])])])]
c4 = Node "a" [(Node "b" []),(Node "c" [])]

i1 = Node 1 []
i2 = Node 2 [i1,i1,i1]
i3 = Node 1 [(Node 2 [(Node 3 []),(Node 4 []),(Node 5 [])]),(Node 6 [(Node 7 [(Node 8 [])])])]

b1 = Leaf "a"
b2 = Fork (Fork (Fork (Leaf "f") (Fork (Fork (Leaf "g") (Leaf "x")) (Leaf "y"))) (Leaf "z")) (Fork (Leaf "h") (Leaf "t"))